###############################################################################################
# $Id: setup_client.py,v 1.21 2004/05/05 20:12:03 chakie Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.surface
import getpass
import traceback
import re
import scenario
import properties
import time
from socket         import *
import gui.widget
from gui.image          import Image
from gui.dialog         import *
from gui.button         import Button
from gui.checkbox       import CheckBox
from gui.tiny_label     import TinyLabel
from gui.title_label    import TitleLabel
from gui.normal_label   import NormalLabel
from gui.editfield      import EditField
from gui.widget_manager import WidgetManager
from gui.messagebox     import Messagebox
from gui.credits        import Credits
from gui.lounge         import Lounge
from gui.lounge_setup   import LoungeSetup


class SetupClient(Dialog):
    """
    This class is used as a dialog for letting the player select a name that is shown to the other
    player. The player can also choose to run the game as fullscreen or windowed
    """

    def __init__ (self):
        "Creates the dialog."
        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # not fullscreen by default
        self.use_fullscreen = 0
        

    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( TitleLabel  ( "Welcome to Civil", (20, 10) ))
        self.wm.register ( NormalLabel ( "My name: ", (300, 220)) )

        # create the version label
        version = "Version %s" % properties.version
        self.wm.register ( TinyLabel ( version , (15, 745) ) )

        # get the user name from the environment if it works
        try:
            name = getpass.getuser()
            try:
                import pwd
                # Try to get the user's first name, as given
                # in the password database
                name = pwd.getpwnam(name)[4].split()[0]
            except:
                # Ignore, and use username
                pass
        except:
            name = "My name"

        self.use_this_name = name
        
        # the editfields
        self.user = EditField ( name, 300, (450, 215) )

        # checkbox
        self.fullscreen = CheckBox ( "Toggle fullscreen mode",
                                      properties.path_dialogs + "butt-radio-set.png",
                                     properties.path_dialogs + "butt-radio-unset.png",
                                     checked=0, position=(290, 295),
                                     callbacks={widget.MOUSEBUTTONUP : self.toggleFullscreen } )
        
        self.wm.register ( self.user )
        self.wm.register ( self.fullscreen )

        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (40, 650 ), {widget.MOUSEBUTTONUP : self.ok } ), K_RETURN )
        self.wm.register ( Button ( properties.path_dialogs + "butt-quit-moff.png",
                                    properties.path_dialogs + "butt-quit-mover.png",
                                    (282, 650 ), {widget.MOUSEBUTTONUP : self.quitCivil } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-credits-moff.png",
                                    properties.path_dialogs + "butt-credits-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.credits } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-lounge-moff.png",
                                    properties.path_dialogs + "butt-lounge-mover.png",
                                    (772, 650 ), {widget.MOUSEBUTTONUP : self.lounge } ) )

        if scenario.commandline_quickstart != 0:
            self.enableTimer(500)
            self.quickstart_state = 0


    def timer(self):
        if 0:
            pass

        elif self.quickstart_state == 0:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONDOWN, { "pos": (320, 370) }))

        elif self.quickstart_state == 1:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONUP,   { "pos": (320, 370) }))

        elif self.quickstart_state == 2:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONDOWN, { "pos": (50, 660) }))

        elif self.quickstart_state == 3:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONUP,   { "pos": (50, 660) }))

        else:
            self.disableTimer()
        
        # increment statemachine
        self.quickstart_state += 1

    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Applies the changes after
        verifying the given data, and closes the dialog. """
 

        # store our name, if we got anything
        if self.user.getText () == "":
            # nothing there, use the login name
            scenario.local_player_name = self.use_this_name
        else:
            scenario.local_player_name = self.user.getText ()
        
        # we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE
   
                
    def quitCivil (self, trigger, event):
        """Callback triggered when the user clicks the 'Quit' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def toggleFullscreen (self, trigger, event):
        """Toggles the fullscreen mode."""
        # toggle fullscreen mode on/off
        pygame.display.toggle_fullscreen ()

        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)
        

    def credits (self, trigger, event):
        """Callback triggered when the user clicks the 'Credits' button. Executes the dialog that
        shows credits information. Then returns to this dialog."""

        # just create a dialog and run it
        Credits ().run ()
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


    def lounge (self, trigger, event):
        """Callback triggered when the user clicks the 'Lounge' button. Executes the dialog that
        contains the lounge date. Then returns to this dialog."""

        # just create a dialog and run it
        dialog = LoungeSetup ()

        if dialog.run () != ACCEPTED:
            # the dialog was cancelled or failed, do nothing
            self.wm.paint (force=1, clear=1)
            return

        # get the connection to the lounge
        connection = dialog.getConnection ()
        
        # accepted, so run the lounge
        Lounge ( connection, self.user.getText () ).run ()
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
