import scenario
from action import Action

class ChangeLOS(Action):
    """
    This class is responsible for changing the line-of-sight between
    units. No point in calculating the information once on the
    server, and then on the clients.
    Plus, the server still has to calculate this information;
    think units that suddenly see each other in a thick forest. Instead
    of just passing by they actually see each other and should start
    shooting.
    """
    def __init__(self, step = -1, unitid = -1, sees = []):
        """Means 'at time _step_, change the army _unitid_ so
        that it sees the armies in _sees_."""
        Action.__init__(self, "change_los", step)
        self.unitid = unitid
        self.sees = sees

    def extract(self, parameters):
        self.step    = int ( parameters[0] )
        self.unitid  = int ( parameters[1] )
        self.sees = map(lambda x: int(x), parameters[2:])

    def execute(self):
        """Executes the action, meaning that we are going to update
        what enemy units the unit sees. Such is life in the trenches."""
        # get the unit with the given id 
        unit = scenario.info.units [ self.unitid ]

        print "execute", self.unitid, self.sees
        to_dispatch = []
        for enemyid in self.sees:
            enemy = scenario.info.units [ enemyid ]
            # Do we still see this enemy?
            if unit.sees.has_key(enemyid):
                assert(enemy.seen_by.has_key(self.unitid))
                continue
            unit.sees[enemyid] = 1
            enemy.seen_by[self.unitid] = 1
            to_dispatch.append(enemy)

        for enemyid in unit.sees.keys(): # ok to change unit.sees now
            enemy = scenario.info.units [ enemyid ]
            assert(enemy.seen_by.has_key(self.unitid))
            if enemyid in self.sees:
                continue
            del unit.sees[enemyid]
            del enemy.seen_by[self.unitid]
            to_dispatch.append(enemy)
            
        # BUG should dispatch some signals?
        # blatant copy-paste
        # make sure the world knows of this change
        # Hmmm, should dispatch later?
        if not scenario.local_player_ai:
            scenario.dispatcher.emit ( 'units_changed', to_dispatch )
            
    def toString(self):
        s = "%s %d %d" % ( self.getName(), self.step, self.unitid)
        for i in self.sees:
            s += " %d" % i
        s += "\n"
        return s

    def __str__(self):
        return self.toString()

    
