###############################################################################################
# $Id: select_theater.py,v 1.1 2004/01/14 07:11:09 chakie Exp $
###############################################################################################

import pygame
import getpass
import traceback
import time

import scenario
import properties
import gui.widget
from gui.image          import Image
from gui.dialog         import *
from gui.button         import Button
from gui.checkbox       import CheckBox
from gui.title_label    import TitleLabel
from gui.normal_label   import NormalLabel
from gui.widget_manager import WidgetManager
from gui.messagebox     import Messagebox
from gui.new_game       import NewGame


class SelectTheater(Dialog):
    """
    This class is used as a dialog for letting the player select which theater to select a scenario
    from. The player is presented with a list of checkboxes from which the wanted theater is
    selected. Stores the selected theater in the properties.
    """

    def __init__ (self):
        "Creates the dialog."
        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )
        

    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( TitleLabel ( "Theater of war",        (20,  10)) )
        self.wm.register ( NormalLabel ( "Select the theater:", (300, 220)) )

        # checkboxes, one for each theater of war
        self.wm.register ( CheckBox ( "US Civil War, 1861-1865", 
                                      properties.path_dialogs + "butt-radio-set.png",
                                      properties.path_dialogs + "butt-radio-unset.png",
                                      checked=1, position=(400, 270),
                                      callbacks={} ) )
        # extra theaters go here...
        

        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (284, 650), {widget.MOUSEBUTTONUP : self.ok } ), K_RETURN )
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (528, 650), {widget.MOUSEBUTTONUP : self.cancel } ) )


    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Finds out the selected theater
        and stores it in the properties."""

        # what theater is selected?
        # theater = self.getTheater ()        
        theater = 'us-civil'

        # TODO: this must be enabled once we start using theatres properly. for now it's all just
        # hardcoded 

        # store the theater where needed
        #self.updateTheater (theater)
        
        # create a dialog and run it
        dialog = NewGame ( theater )
        state = dialog.run ()

        # was it accepted ?
        if state == ACCEPTED:
            # we're accepting the dialog
            self.state = ACCEPTED
        
        return widget.DONE

   
                
    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def updateTheater (self, theater):
        """Updates all data that depends on the theater. This is mainly some properties paths. This
        method creates full paths to some resources that has to be loaded depending on the theater."""
        # update the paths
        properties.path_units     = properties.path_gfx + '/units/'     + theater + '/'
        properties.path_periphery = properties.path_gfx + '/periphery/' + theater + '/'
        properties.path_cursors   = properties.path_gfx + '/pointers/'  + theater + '/'


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
