###############################################################################################
# $Id: timeout.py,v 1.3 2003/11/26 21:24:51 chakie Exp $
###############################################################################################

import scenario
import constants

from server.engine.ai.module import Module
from action.end_game_act     import EndGameAct


class Timeout (Module):
    """This class defines a module that when executed checks the current turn, and if it is as large
    as the max allowed turn for the scenario then the game has ended. The game ends by sending out
    an EndGameAct ation to the clients.
    
    This module is executed once every turn, on the last step.
    """


    def __init__ (self):
        """Initializes the module. Sets a suitable name."""

        # call superclass with the delays
        Module.__init__ (self, 'timeout' )


    def isReady (self, step):
        """Checks wether the module is ready to be executed. This module is ready on the last step
        of the action."""

        # is this the last turn?
        if step == scenario.time.getMaxStep ():
            # we're ready now!
            return 1

        # not yet last turn
        return 0


    def execute (self, step):
        """Executes the module. Checks the current turn and if it is too large, i.e. larger than the
        max turns then the game is ended."""

        # no action yet
        actiondata = []

        # have we advanced to the end already?
        if scenario.time.getTurn () >= scenario.info.getMaxTurns ():
            # game is over
            actiondata.append ( EndGameAct ( constants.TIMEOUT, step ) )
        
        return actiondata
    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
